-----------------------
--- Module Creation ---
-----------------------
local core = LibStub("AceAddon-3.0"):GetAddon("tomQuest2")
local module = core:NewModule("soundNotifications", "AceEvent-3.0")

-----------------------
--- Library loading ---
-----------------------
local LSM = LibStub("LibSharedMedia-3.0")

---------------------------------------
--- Shared Media Sound Registration ---
---------------------------------------
LSM:Register("sound", "Peon Ready", [[Sound\Creature\Peon\PeonReady1.wav]])
LSM:Register("sound", "Work Done", [[Interface\Addons\tomQuest2\media\Peasant_work_done.mp3]])
LSM:Register("sound", "Thrumm", [[Sound\Interface\igPlayerBind.wav]])
LSM:Register("sound", "More work?", [[Sound\Creature\Peasant\PeasantWhat3.wav]])
LSM:Register("sound", "Work complete!", [[Sound\Creature\Peon\PeonBuildingComplete1.wav]])
LSM:Register("sound", "Quest Added", [[Sound\interface\iQuestActivate.wav]])
LSM:Register("sound", "Quest Completed", [[Sound\interface\iQuestComplete.wav]])
LSM:Register("sound", "Quest Failed", [[Sound\interface\igQuestFailed.wav]])
LSM:Register("sound", "Auction Window Open", [[Sound\interface\AuctionWindowOpen.wav]])
LSM:Register("sound", "Auction Window Close", [[Sound\interface\AuctionWindowClose.wav]])
LSM:Register("sound", "Ready Check", [[Sound\interface\levelup2.wav]])
LSM:Register("sound", "Level Up", [[Sound\interface\levelup.wav]])

--------------------------------------------
--- Copy global functions to local scope ---
--------------------------------------------
local PlaySoundFile = PlaySoundFile
local PlaySound = PlaySound

-----------------------
--- Default profile ---
-----------------------
local defaults = {
	profile = {
		soundQuestComplete = "Work Done",
		soundQuestFailed = "Thrumm",
		soundObjectiveComplete = "Peon Ready",
	},
}

----------------------
--- Initialization ---
----------------------
function module:OnInitialize()
	self.db = core.db:RegisterNamespace("soundNotifications", defaults, "Default")
	self:SetEnabledState(core:getModuleEnabled("soundNotifications"))
	core.moduleState["soundNotifications"] = core:getModuleEnabled("soundNotifications")
end

function module:OnEnable()
	core.RegisterCallback(self, "playerQuestObjectiveProgress")
	core.RegisterCallback(self, "playerQuestComplete")
	core.RegisterCallback(self, "playerQuestFailed")
end

function module:OnDisable()
	core.UnregisterAllCallbacks(self)
end

local function Play(sound)
	local soundPlayed = PlaySoundFile(sound)
	if not soundPlayed then
		PlaySound(sound)
	end
end

---------------------------
--- Main Module Program ---
---------------------------
function module:playerQuestObjectiveProgress(event, commHandler, uid, objective, got, nGot)
	if nGot == core.questsObjectives[uid][objective].nNeed and core.quests[uid].complete[core.playerName] == 0 then
		Play(LSM:Fetch("sound", self.db.profile.soundObjectiveComplete))
	end
end

function module:playerQuestComplete(event, commHandler, uid)
	Play(LSM:Fetch("sound", self.db.profile.soundQuestComplete))
end

function module:playerQuestFailed(event, commHandler, uid)
	Play(LSM:Fetch("sound", self.db.profile.soundQuestFailed))
end